/**
 * Global state for Telemetry instance (survives hot reloads)
 */
const telemetryInstanceKey = Symbol.for('@agentuity/telemetry:instance');
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const g = globalThis;
export const telemetry = {
    get: () => g[telemetryInstanceKey],
    set: (v) => {
        g[telemetryInstanceKey] = v;
    },
};
//# sourceMappingURL=globals.js.map