/**
 * @agentuity/telemetry - OpenTelemetry telemetry for Agentuity
 *
 * Auto-initializes from environment variables on import (Vercel-style).
 *
 * @example Automatic initialization (recommended)
 * ```typescript
 * // Just import - auto-configures from AGENTUITY_* env vars
 * import '@agentuity/telemetry';
 *
 * // Then access the globals anywhere
 * import { tracer, logger, meter } from '@agentuity/telemetry';
 * ```
 *
 * @example Explicit configuration
 * ```typescript
 * import { register } from '@agentuity/telemetry';
 *
 * register({
 *   name: 'my-app',
 *   version: '1.0.0',
 *   // ... optional overrides
 * });
 * ```
 */
export type { Logger } from './logger';
export { __originalConsole } from './logger';
export type { TelemetryConfig, TelemetryResponse } from './telemetry';
export { injectTraceContextToHeaders, extractTraceContextFromRequest } from './http';
export { enrichContextWithTraceState, generateTraceId, generateSpanId, type TraceStateEntries, } from './tracestate';
export { register, registerTelemetry, getTelemetry, ensureInitialized } from './telemetry';
import type { Tracer, Meter } from '@opentelemetry/api';
import type { Logger } from './logger';
/**
 * Get the OpenTelemetry tracer (auto-initialized)
 */
export declare const tracer: Tracer;
/**
 * Get the OpenTelemetry meter (auto-initialized)
 */
export declare const meter: Meter;
/**
 * Get the Logger instance (auto-initialized)
 */
export declare const logger: Logger;
/**
 * Shutdown telemetry (call on process exit)
 */
export declare function shutdown(): Promise<void>;
//# sourceMappingURL=index.d.ts.map